%Replicate empirical results for data from Angrist Krueger (1991), reported in
%Andrews and Armstrong (2016).
%Specifications as in Staiger and Stock (1997)
clear

%set random number generator
rng(1);

%Use homoskedastic or heteroskedastic version of robust statistics
heterosked_robust=1;
RB_draws=10^5;
%Specify grid of values to consider for c
c_grid=[0,0.1,0.5,0.9];
%Choose which speficiation to use: corresponds to column number in Andrews and Armstrong (2016)
spec=4;

load AK_3039_analysis_dataset;
st_dataset(end,:)=[];

AGEQ=double(st_dataset(1:end,2));
AGEQSQ=double(st_dataset(1:end,28));
EDUC=double(st_dataset(1:end,4));
ENOCENT=double(st_dataset(1:end,5));
ESOCENT=double(st_dataset(1:end,6));
LWKLYWGE=double(st_dataset(1:end,8));
MARRIED=double(st_dataset(1:end,9));
MIDATL=double(st_dataset(1:end,10));
MT=double(st_dataset(1:end,11));
NEWENG=double(st_dataset(1:end,12));
RACE=double(st_dataset(1:end,18));
SMSA=double(st_dataset(1:end,19));
SOATL=double(st_dataset(1:end,20));
WNOCENT=double(st_dataset(1:end,23));
WSOCENT=double(st_dataset(1:end,24));
YR20_28=double(st_dataset(1:end,29:37));
QTR120_129=double(st_dataset(1:end,43:52));
QTR220_229=double(st_dataset(1:end,53:62));
QTR320_329=double(st_dataset(1:end,63:72));
QTR1_3=double(st_dataset(1:end,39:41));
v17=double(st_dataset(1:end,16));

SOB_index=[1 2 4 5 6 8 9 10 11 12 13 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30 31 ...
    32 33 34 35 36 37 38 39 40 41 42 44 45 46 47 48 49 50 51 53 54 55]; %drop 56 to avoid multicollinartity
SOB_dummies=zeros(size(QTR1_3,1),length(SOB_index));
for n=1:length(SOB_index)
    SOB_dummies(:,n)=double(v17==SOB_index(n));
end

%%
%Test for heteroskedasticity in first-stage residuals based on QOB
Z=[ones(size(QTR1_3,1),1)-sum(QTR1_3,2) QTR1_3];
X=EDUC;
Y=LWKLYWGE;

%Special structure in this application means that columns of Z are
%orthogonal, so need only a subset of Psi from White 1980.
Psi=Z(:,2:end);

b=(Z'*Z)^-1*Z'*X;
e_hatX=X-Z*b;
sigma_hat=mean(e_hatX.^2);
D=mean(Psi.*repmat(e_hatX.^2-sigma_hat,1,size(Psi,2)));
B_base=(Psi-repmat(mean(Psi),size(Psi,1),1)).*repmat(e_hatX.^2-sigma_hat,1,size(Psi,2));
B=B_base'*B_base/size(Z,1);
White_statX=size(Z,1)*D*B^-1*D';

b=(Z'*Z)^-1*Z'*Y;
e_hatY=Y-Z*b;
sigma_hat=mean(e_hatY.^2);
D=mean(Psi.*repmat(e_hatY.^2-sigma_hat,1,size(Psi,2)));
B_base=(Psi-repmat(mean(Psi),size(Psi,1),1)).*repmat(e_hatY.^2-sigma_hat,1,size(Psi,2));
B=B_base'*B_base/size(Z,1);
White_statY=size(Z,1)*D*B^-1*D';

sigma_hat=mean(e_hatX.*e_hatY);
D=mean(Psi.*repmat(e_hatX.*e_hatY-sigma_hat,1,size(Psi,2)));
B_base=(Psi-repmat(mean(Psi),size(Psi,1),1)).*repmat(e_hatY.^2-sigma_hat,1,size(Psi,2));
B=B_base'*B_base/size(Z,1);
White_statXY=size(Z,1)*D*B^-1*D';

White_stat_mat=[White_statY White_statXY; White_statXY White_statX];

%%

%Outcome variable is wages, endogenous variable is education, Z is
%instruments, and W are controls
Y=LWKLYWGE;
X=EDUC;

%Collect exogenous control variables and instruments.  Have checked that
%results in 1-3 match corresponding results in Staiger and Stock.
if spec==1
    Z=QTR1_3;
    W=[ones(length(Y),1) RACE MARRIED SMSA NEWENG MIDATL ENOCENT WNOCENT SOATL ESOCENT WSOCENT MT YR20_28];
elseif spec==2
    W=[ones(length(Y),1) RACE MARRIED SMSA NEWENG MIDATL ENOCENT WNOCENT SOATL ESOCENT WSOCENT MT YR20_28];
    Z=[QTR120_129 QTR220_229 QTR320_329];
elseif spec==3
    W=[ones(length(Y),1) RACE MARRIED SMSA NEWENG MIDATL ENOCENT WNOCENT SOATL ESOCENT WSOCENT MT AGEQ AGEQSQ YR20_28];
    Z=[QTR120_129 QTR220_229 QTR320_329];
    Z(:,end)=[];
    Z(:,end-1)=[];
else
    W=[ones(length(Y),1) RACE MARRIED SMSA NEWENG MIDATL ENOCENT WNOCENT SOATL ESOCENT WSOCENT MT AGEQ AGEQSQ YR20_28 SOB_dummies];
    Z=[QTR120_129 QTR220_229 QTR320_329];
    Z(:,end)=[];
    Z(:,end-1)=[];
    Z=[Z repmat(SOB_dummies,1,3).*repmat(QTR1_3,1,size(SOB_dummies,2))];
end

%Drop rows with missing observations
data=[Y X Z W];
nanrows=isnan(sum(data,2));
clear data SOB_index AGEQ AGEQSQ EDUC ENOCENT ESOCENT LWKLYWGE MARRIED QTR220_229 QTR1_3 st_dataset
clear MIDATL MT NEWENG RACE SMSA SOATL WNOCENT WSOCENT YR20_28 QTR120_129 QTR320_329 SOB_dummies v17
Y=Y(nanrows==0,:);
X=X(nanrows==0,:);
Z=Z(nanrows==0,:);
W=W(nanrows==0,:);

%Run as 2SLS with included exogenous variables
X_tilde=[X W];
Z_tilde=[Z W];

ZZ=Z_tilde'*Z_tilde;
ZZ_inv=ZZ^-1;
ZX=Z_tilde'*X_tilde;
XZ=X_tilde'*Z_tilde;
ZY=Z_tilde'*Y;

beta_tilde=(XZ*ZZ_inv*ZX)^-1*XZ*ZZ_inv*ZY; %Matches estimate from AK code and paper

WW=W'*W;
WZ=W'*Z;
WX=W'*X;
WY=W'*Y;

Y=Y-W*WW^-1*WY;
X=X-W*WW^-1*WX;
Z=Z-W*WW^-1*WZ;

ZZ=Z'*Z;
ZZ_inv=ZZ^-1;
ZX=Z'*X;
XZ=X'*Z;
ZY=Z'*Y;

beta_hat_2SLS=(XZ*ZZ_inv*ZX)^-1*XZ*ZZ_inv*ZY;
g_hat=IV_moment_condition(beta_hat_2SLS,Y,X,Z);
Sigma_hat=Heterosked_robust_covariance_estimate(g_hat);
Var_estimator=length(Z)*(XZ*ZZ_inv*ZX)^-1*XZ*ZZ_inv*Sigma_hat*ZZ_inv*ZX*(XZ*ZZ_inv*ZX)^-1;
se_beta_hat=Var_estimator^.5;
Sigma_hat_homosked=1/length(Y)*ZZ*mean((Y-X*beta_hat_2SLS).^2);
Var_estimator_homosked=length(Z)*(XZ*ZZ_inv*ZX)^-1*XZ*ZZ_inv*Sigma_hat_homosked*ZZ_inv*ZX*(XZ*ZZ_inv*ZX)^-1;

obs=length(Y);
instruments=size(Z,2);
g=@(b) ZY-ZX*b;
qmat=repmat(X,1,instruments).*Z;
qmat_sum=sum(qmat);

clear W X_tilde Z_tilde

if heterosked_robust==1
    omega_base=[repmat(Y,1,instruments).*Z repmat(X,1,instruments).*Z];
    Omega=Heterosked_robust_covariance_estimate(omega_base);
    Omega11=Omega(1:instruments,1:instruments);
    Omega12=Omega(1:instruments,instruments+1:2*instruments);
    Omega21=Omega(instruments+1:2*instruments,1:instruments);
    Omega22=Omega(instruments+1:2*instruments,instruments+1:2*instruments);
else
    b1=ZZ_inv*Z'*Y;
    V1=Y-Z*b1;
    b2=ZZ_inv*ZX;
    V2=X-Z*b2;
    cov_mat=cov([V1 V2]);
    Omega=kron(cov_mat,1/obs*(Z'*Z));
    Omega11=Omega(1:instruments,1:instruments);
    Omega12=Omega(1:instruments,instruments+1:2*instruments);
    Omega21=Omega(instruments+1:2*instruments,1:instruments);
    Omega22=Omega(instruments+1:2*instruments,instruments+1:2*instruments);
end

Sigma_gg=@(b) (Omega11-b*(Omega12+Omega21)+b^2*Omega22);
Sigma_tg=@(b) (Omega21-b*Omega22);
Sigma_tt=@(b) Omega22;

%Calculate LIML
l=@(b)1/obs*(g(b)'*ZZ_inv*g(b))/((Y-X*b)'*(Y-X*b));
LIML=fminsearch(l,beta_hat_2SLS);

%Calculate Fuller as in Hansen, Hausman, and Newey (2008)
C=1;
P1=ZZ_inv*Z';
Xbar=[Y X];
alpha_tilde=min(eig((Xbar'*Xbar)^-1*(Xbar'*Z)*ZZ_inv*(Z'*Xbar)));
alpha_hat=(alpha_tilde-(1-alpha_tilde)*C/obs)/(1-(1-alpha_tilde)*C/obs);
FULL=(ZX'*ZZ_inv*ZX-alpha_hat*(X'*X))^-1*(ZX'*ZZ_inv*Z'*Y-alpha_hat*(X'*Y));

u=@(delta) Y-X*delta;
sigma2_u=@(delta) u(delta)'*u(delta)/(obs-1);
alpha_tilde=@(delta) (u(delta)'*Z)*ZZ_inv*(Z'*u(delta))/(u(delta)'*u(delta));
Upsilon=Z*ZZ_inv*(ZX);
X_tilde=@(delta) X-u(delta)*(u(delta)'*X)/(u(delta)'*u(delta));
V=@(delta) X_tilde(delta)-Z*ZZ_inv*(Z'*X_tilde(delta));
P_diag=sum(Z.*P1',2);
kappa=sum(P_diag.^2)/instruments;
tau=instruments/obs;

H_hat=@(delta) (ZX')*ZZ_inv*(ZX)-alpha_tilde(delta)*(X'*X);
Sigma_hat_B=@(delta) sigma2_u(delta)*((1-alpha_tilde(delta))^2*...
    (X_tilde(delta)'*Z)*ZZ_inv*(Z'*X_tilde(delta))+alpha_tilde(delta)^2*X_tilde(delta)'*V(delta));
A_hat=@(delta) sum((P_diag-tau).*Upsilon)*sum(u(delta).^2.*V(delta)/obs);
B_hat=@(delta) instruments*(kappa-tau)*sum((u(delta).^2-sigma2_u(delta)).*V(delta).^2)...
    /(obs*(1-2*tau+kappa*tau));
Sigma_hat=@(delta) Sigma_hat_B(delta)+2*A_hat(delta)+B_hat(delta);
Var_estimator=H_hat(FULL)^-1*Sigma_hat(FULL)*H_hat(FULL)^-1;
se_FULL=Var_estimator^.5;

beta_FULL=FULL;

clear u sigma2_u alpha_tilde Upsilon X_tilde P1 P_diag A_hat H_hat B_hat Sigma_hat_B Sigma_hat Xbar_scaled
clear X_dot X_hat e_hat H_hat Z_tilde omega_base

%%
%Calculate First Stage F statistic
pi_hat=ZZ_inv*Z'*X;
V_hat=X-Z*pi_hat;

%First stage F statistic assuming homoskedastic errors
Sigma_VV=(V_hat'*V_hat-sum(V_hat)^2)/(obs-instruments);
F_stat_homosked=Sigma_VV^-1*XZ*ZZ_inv*ZX/instruments;

%First Stage F statistic assuming heteroskedastic errors: reported in paper
e_base=Z.*repmat(V_hat,1,size(Z,2));
Sigma_pi=ZZ*((e_base-repmat(mean(e_base,1),obs,1))'*...
    (e_base-repmat(mean(e_base,1),obs,1)))^-1*ZZ;
F_stat_heterosked=pi_hat'*Sigma_pi*pi_hat/instruments;

clear e_base V_hat
%%
%calculate baseline unbiased estimates

ZZ=Z'*Z;
ZZ_inv=ZZ^-1;
ZX=Z'*X;
XZ=X'*Z;
ZY=Z'*Y;

%Excluded group is forth quarter births, so first-stage coefficients are
%expected to be negative.
xi1=ZZ_inv*ZY;
U_hat=Y-Z*xi1;
xi2=ZZ_inv*ZX;
V_hat=X-Z*xi2;
xi=[xi1;xi2];
e_base=[Z.*repmat(U_hat,1,size(Z,2)) Z.*repmat(V_hat,1,size(Z,2))];
Sigma=kron(eye(2),ZZ_inv)*((e_base-repmat(mean(e_base,1),obs,1))'*...
    (e_base-repmat(mean(e_base,1),obs,1)))*kron(eye(2),ZZ_inv);
Sigma11=Sigma(1:instruments,1:instruments);
Sigma12=Sigma(1:instruments,instruments+1:end);
Sigma22=Sigma(instruments+1:end,instruments+1:end);
clear e_base X Y Z

sigma11_vec=diag(Sigma11).^.5;
sigma12_vec=diag(Sigma12);
sigma22_vec=diag(Sigma22).^.5;

sigma2_tilde=2^.5*sigma22_vec;
sigma1_tilde=2^.5*sigma11_vec;
sigma12_tilde=2*sigma12_vec;
sigma2_tilde_mat=repmat(sigma2_tilde,1,RB_draws);
sigma12_tilde_mat=repmat(sigma12_tilde,1,RB_draws);
ZZ_rt=ZZ^.5;

zeta=randn(instruments*2,RB_draws);
zeta=Sigma^.5*zeta;
xi_a=repmat(xi,1,RB_draws)+zeta;
xi_b=repmat(xi,1,RB_draws)-zeta;
clear zeta
xi1_a=xi_a(1:instruments,:);
xi2_a=xi_a(instruments+1:end,:);
xi2_b=xi_b(instruments+1:end,:);
clear xi_a xi_b
tau=-pi^.5./(2^.5*sigma2_tilde_mat).*exp(xi2_a.^2./(2*sigma2_tilde_mat.^2)).*erfc(-xi2_a./(2^.5*sigma2_tilde_mat));
beta_hat=tau.*(xi1_a-sigma12_tilde_mat./sigma2_tilde_mat.^2.*xi2_a)+sigma12_tilde_mat./sigma2_tilde_mat.^2;
%Substitute 2SLS for unbiased estimator in cases where cannot
%compute unbiased estimate.
elementwise_IV_mat=xi1_a./xi2_a;
beta_hat(isnan(beta_hat))=elementwise_IV_mat(isnan(beta_hat));
beta_hat(isinf(beta_hat))=elementwise_IV_mat(isinf(beta_hat));

w=(ZZ*xi2_b).*xi2_b;
w=w./repmat(sum(w,1),instruments,1);
beta_hat_RB=mean(sum(w.*beta_hat,1));


%%
%Calculate unbiased estimates based on transformations of instruments

%construct matricies useful for estimation based on transformations of the
%instruments
for m2=1:length(c_grid)
    c=c_grid(m2);
    A_c=ones(instruments,instruments)*c+eye(instruments)*(1-c);
    temp=diag(diag(Sigma22))^-.5*Sigma22*diag(diag(Sigma22))^-.5;
    w2_store(:,:,m2)=diag(diag(A_c*temp*A_c))^-.5*A_c*diag(diag(Sigma22))^-.5;
    w2_inv_store(:,:,m2)=w2_store(:,:,m2)^-1;
    w2=w2_store(:,:,m2);
    
    sigma2t_vec=diag(2*w2*Sigma22*w2').^.5;
    sigma12t_vec=diag(2*w2*Sigma12*w2');
    sigma2t_mat(:,:,m2)=repmat(sigma2t_vec,1,RB_draws);
    sigma12t_mat(:,:,m2)=repmat(sigma12t_vec,1,RB_draws);
end

beta_hat_RB_t=zeros(length(c_grid),1);
for m2=1:length(c_grid)
    w2=w2_store(:,:,m2);
    w2_inv=w2_inv_store(:,:,m2);
    
    xi1_a_t=w2*xi1_a;
    xi2_a_t=w2*xi2_a;
    xi2_b_t=w2*xi2_b;
    
    tau=-pi^.5./(2^.5*sigma2t_mat(:,:,m2)).*exp(xi2_a_t.^2./(2*sigma2t_mat(:,:,m2).^2)).*erfc(-xi2_a_t./(2^.5*sigma2t_mat(:,:,m2)));
    beta_hat=tau.*(xi1_a_t-sigma12t_mat(:,:,m2)./(sigma2t_mat(:,:,m2).^2).*xi2_a_t)+sigma12t_mat(:,:,m2)./(sigma2t_mat(:,:,m2).^2);
    w=(w2_inv'*ZZ*w2_inv*xi2_b_t).*(xi2_b_t);
    w=w./repmat(sum(w,1),instruments,1);
    %Substitute 2SLS for unbiased estimator in cases where cannot
    %compute unbiased estimate.
    elementwise_IV_mat=xi1_a_t./xi2_a_t;
    beta_hat(isnan(beta_hat))=elementwise_IV_mat(isnan(beta_hat));
    beta_hat(isinf(beta_hat))=elementwise_IV_mat(isinf(beta_hat));
    beta_hat_RB_t(m2)=mean(sum(w.*beta_hat,1));
end


%Calculate ID-Robust CS
beta_grid=[sort(-(1:1:100).^2) -1:0.001:1 (1:1:100).^2];
size_sims=10^5;
S_store=zeros(length(beta_grid),1);
K_store=zeros(length(beta_grid),1);
rk_store=zeros(length(beta_grid),1);
QCLR_crit_store=zeros(length(beta_grid),1);
K_draw=sum(randn(size_sims,1).^2,2);
J_draw=sum(randn(size_sims,instruments).^2,2);
for m=1:length(beta_grid)
    m/length(beta_grid);
    b_temp=beta_grid(m);
    Sigma_gg_temp=Sigma_gg(b_temp);
    Sigma_gg_temp_inv=Sigma_gg_temp^-1;
    Sigma_tg_temp=Sigma_tg(b_temp);
    Sigma_tt_temp=Sigma_tt(b_temp);
 
    g_temp=g(b_temp);    
    S_temp=1/obs*g_temp'*Sigma_gg_temp^-1*g_temp;    
    D_temp=qmat_sum'-Sigma_tg_temp*Sigma_gg_temp^-1*g_temp;
    K_temp=1/obs*g_temp'*Sigma_gg_temp_inv*D_temp*...
        (D_temp'*Sigma_gg_temp_inv*D_temp)^-1*D_temp'*Sigma_gg_temp_inv*g_temp;    
    S_store(m)=S_temp;
    K_store(m)=K_temp;
    
    sigma_D_temp=Sigma_tt_temp-Sigma_tg_temp'*Sigma_gg_temp^-1*Sigma_tg_temp;
    rk_store(m)=1/obs*D_temp'*sigma_D_temp^-1*D_temp;
    QCLR_crit_store(m)=quantile(.5*(K_draw+J_draw-rk_store(m)+((K_draw+J_draw+rk_store(m)).^2-4*rk_store(m).*J_draw).^.5),.95);
end

QCLR_store=.5*(S_store-rk_store+((S_store+rk_store).^2-4*rk_store.*(S_store-K_store)).^.5);
QCLR_CI=QCLR_store<QCLR_crit_store;

plot(beta_grid,QCLR_CI)