%Replicate empirical results for data from Hornung (2014), reported in
%Andrews and Armstrong (2016).

%Run data cleaning file
Clean_data;

%Generate Regressor List as in Hornung
mixlist=[ln_workers1802_all ln_looms1802_all mi_ln_input1802_all no_looms ln_popcity1802 vieh1816_schaf_ganz_veredelt_pc pop1816_prot_pc no_hugue_possible imputed_dummy];

%betagrid=[0.01:0.01:100 100:10:1000 10^6];
%betagrid=sort([betagrid 0 -betagrid]);
betagrid=-100:0.01:100;

%%
%Calculate estimates as in Table 4, Column 3
W=[ones(size(poploss_keyser,1),1) textil_1680_dummy mixlist];
X=hugue_1700_pc;
Z=poploss_keyser;
Y=ln_output1802_all;

temp=[Y X W Z];
nanrows=sum(isnan(temp),2)>0;
Y=Y(nanrows==0,:);
X=X(nanrows==0,:);
W=W(nanrows==0,:);
Z=Z(nanrows==0,:);
ID=groupid(nanrows==0);
obs=length(Y);

%Partial out control variables
M_W=eye(size(W,1))-W*(W'*W)^-1*W';
Y=M_W*Y;
X=M_W*X;
Z=M_W*Z;

xi1=(Z'*Z)^-1*Z'*Y;
xi2=(Z'*Z)^-1*Z'*X;

U_hat=Y-Z*xi1;
V_hat=X-Z*xi2;
Omega=Clustered_covariance_estimate([U_hat V_hat].*repmat(Z,1,2),ID);
%Mimic Stata's degrees of freedom correction
Omega=Omega*(57/56)*(obs)/(obs-size(W,2));
Sigma=obs*kron(eye(2),(Z'*Z)^-1)*Omega*kron(eye(2),(Z'*Z)^-1);
sigma1=Sigma(1,1)^.5;
sigma12=Sigma(1,2);
sigma2=Sigma(2,2)^.5;

tau=pi^.5/(2^.5*sigma2)*exp(xi2.^2/(2*sigma2^2)).*erfc(xi2/(2^.5*sigma2));
beta_hat_U=tau.*(xi1-sigma12/sigma2^2*xi2)+sigma12/sigma2^2;
beta_hat_2SLS=xi1/xi2;
beta_hat_Full=(xi1.*xi2+sigma12)./(xi2.^2+sigma2^2);
F_stat=xi2^2/sigma2^2;

Results=[beta_hat_U beta_hat_2SLS beta_hat_Full F_stat];

b=betagrid;
AR_stack=(xi1-b*xi2).^2./(sigma1^2-2*b*sigma12+b.^2*sigma2^2);

%%
%Calculate Estimates as in Table 4, Column 5
W=[ones(size(poploss_keyser,1),1) textil_1680_dummy mixlist];
X=hugue_1700_pc;
Z=poploss_keyser_projected;
Y=ln_output1802_all;

temp=[Y X W Z];
nanrows=sum(isnan(temp),2)>0;
Y=Y(nanrows==0,:);
X=X(nanrows==0,:);
W=W(nanrows==0,:);
Z=Z(nanrows==0,:);
ID=groupid(nanrows==0);
obs=length(Y);

%Partial out control variables
M_W=eye(size(W,1))-W*(W'*W)^-1*W';
Y=M_W*Y;
X=M_W*X;
Z=M_W*Z;

xi1=(Z'*Z)^-1*Z'*Y;
xi2=(Z'*Z)^-1*Z'*X;

U_hat=Y-Z*xi1;
V_hat=X-Z*xi2;
Omega=Clustered_covariance_estimate([U_hat V_hat].*repmat(Z,1,2),ID);
%Mimic Stata's degrees of freedom correction
Omega=Omega*(57/56)*(obs)/(obs-size(W,2));
Sigma=obs*kron(eye(2),(Z'*Z)^-1)*Omega*kron(eye(2),(Z'*Z)^-1);
sigma1=Sigma(1,1)^.5;
sigma12=Sigma(1,2);
sigma2=Sigma(2,2)^.5;

tau=pi^.5/(2^.5*sigma2)*exp(xi2.^2/(2*sigma2^2)).*erfc(xi2/(2^.5*sigma2));
beta_hat_U=tau.*(xi1-sigma12/sigma2^2*xi2)+sigma12/sigma2^2;
beta_hat_2SLS=xi1/xi2;
beta_hat_Full=(xi1.*xi2+sigma12)./(xi2.^2+sigma2^2);
F_stat=xi2^2/sigma2^2;

Results(2,:)=[beta_hat_U beta_hat_2SLS beta_hat_Full F_stat];

b=betagrid;
AR_stack(2,:)=(xi1-b*xi2).^2./(sigma1^2-2*b*sigma12+b.^2*sigma2^2);

%%
%Calculate Estimates as in Table 5, Column 3
W=[ones(size(poploss_keyser,1),1) textil_1680_dummy mixlist];
X=hugue_1700_pc;
Z=poploss_aggregated_pr;
Y=ln_output1802_all;

temp=[Y X W Z];
nanrows=sum(isnan(temp),2)>0;
Y=Y(nanrows==0,:);
X=X(nanrows==0,:);
W=W(nanrows==0,:);
Z=Z(nanrows==0,:);
ID=groupid(nanrows==0);
obs=length(Y);

%Partial out control variables
M_W=eye(size(W,1))-W*(W'*W)^-1*W';
Y=M_W*Y;
X=M_W*X;
Z=M_W*Z;

xi1=(Z'*Z)^-1*Z'*Y;
xi2=(Z'*Z)^-1*Z'*X;

U_hat=Y-Z*xi1;
V_hat=X-Z*xi2;
Omega=Clustered_covariance_estimate([U_hat V_hat].*repmat(Z,1,2),ID);
%Mimic Stata's degrees of freedom correction
Omega=Omega*(71/70)*(obs)/(obs-size(W,2));
Sigma=obs*kron(eye(2),(Z'*Z)^-1)*Omega*kron(eye(2),(Z'*Z)^-1);
sigma1=Sigma(1,1)^.5;
sigma12=Sigma(1,2);
sigma2=Sigma(2,2)^.5;

tau=pi^.5/(2^.5*sigma2)*exp(xi2.^2/(2*sigma2^2)).*erfc(xi2/(2^.5*sigma2));
beta_hat_U=tau.*(xi1-sigma12/sigma2^2*xi2)+sigma12/sigma2^2;
beta_hat_2SLS=xi1/xi2;
beta_hat_Full=(xi1.*xi2+sigma12)./(xi2.^2+sigma2^2);
F_stat=xi2^2/sigma2^2;

Results(3,:)=[beta_hat_U beta_hat_2SLS beta_hat_Full F_stat];

b=betagrid;
AR_stack(3,:)=(xi1-b*xi2).^2./(sigma1^2-2*b*sigma12+b.^2*sigma2^2);
%%
%Calculate Estimates as in Table 5, Column 6
W=[ones(size(poploss_keyser,1),1) textil_1680_dummy mixlist];
X=ln_occ1700_hugue_textile;
Z=poploss_aggregated_pr;
Y=ln_output1802_all;

temp=[Y X W Z];
nanrows=sum(isnan(temp),2)>0;
Y=Y(nanrows==0,:);
X=X(nanrows==0,:);
W=W(nanrows==0,:);
Z=Z(nanrows==0,:);
ID=groupid(nanrows==0);
obs=length(Y);

%Partial out control variables
M_W=eye(size(W,1))-W*(W'*W)^-1*W';
Y=M_W*Y;
X=M_W*X;
Z=M_W*Z;

xi1=(Z'*Z)^-1*Z'*Y;
xi2=(Z'*Z)^-1*Z'*X;

U_hat=Y-Z*xi1;
V_hat=X-Z*xi2;
Omega=Clustered_covariance_estimate([U_hat V_hat].*repmat(Z,1,2),ID);
%Mimic Stata's degrees of freedom correction
Omega=Omega*(71/70)*(obs)/(obs-size(W,2));
Sigma=obs*kron(eye(2),(Z'*Z)^-1)*Omega*kron(eye(2),(Z'*Z)^-1);
sigma1=Sigma(1,1)^.5;
sigma12=Sigma(1,2);
sigma2=Sigma(2,2)^.5;

tau=pi^.5/(2^.5*sigma2)*exp(xi2.^2/(2*sigma2^2)).*erfc(xi2/(2^.5*sigma2));
beta_hat_U=tau.*(xi1-sigma12/sigma2^2*xi2)+sigma12/sigma2^2;
beta_hat_2SLS=xi1/xi2;
beta_hat_Full=(xi1.*xi2+sigma12)./(xi2.^2+sigma2^2);
F_stat=xi2^2/sigma2^2;

Results(4,:)=[beta_hat_U beta_hat_2SLS beta_hat_Full F_stat];

b=betagrid;
AR_stack(4,:)=(xi1-b*xi2).^2./(sigma1^2-2*b*sigma12+b.^2*sigma2^2);

CI_stack=AR_stack<=chi2inv(.95,1);
plot(betagrid,CI_stack(4,:))