%Simulate performance of unbiased estimators, relative to risk envelope, in
%imulation calibration assuming homoskedasticity
clear
rng(1);
%set which calibration to use
load AK_sim_calibration_SS_spec_3

parpool('local',24)

sims=10^6;
RB_draws=10^3;

%Set range of mean for firs-stage F statistic over which to plot
Lower_F=2;
Upper_F=24.5;

%Set number of grid points for pi at which to check performance
pi_grid_points=15;

%Set values which values c to consider for estimation based on linear
%transformations of the instruments
c_grid=0.1:0.4:0.9;

%Set grid of values rho to consider
rho_grid=[0.1 0.5 0.95];

%set grid of quantile values for which to store estimator
quantile_grid=(0.01:0.01:0.99);

%set sign restriction on pi coefficints
pi_negative=1;
if pi_negative==1
    pi_hat_2SLS=-pi_hat_2SLS;
end

ZZ_inv=ZZ^-1;

instruments=length(pi_hat_2SLS);

Sigma22=obs*ZZ_inv*sigmaV^2;

%Estimate pi enforcing restriction that positive.  Motivated as posterior
%mean of pi for a flat prior supported on the positive orthant.
sigma2_vec=diag(Sigma22).^.5;
pi_true=pi_hat_2SLS+sigma2_vec.*(normpdf(-pi_hat_2SLS./(sigma2_vec)))...
    ./(1-normcdf(-pi_hat_2SLS./(sigma2_vec)));
%Normalize length of pi_hat to one
pi_true=pi_true/norm(pi_true);

%Set scaling for pi grid taking into account homoskedasticity restriction
F_true_mean=pi_true'*ZZ*pi_true/instruments+1;
min_scaling=((Lower_F-1)./(F_true_mean-1))^.5;
max_scaling=((Upper_F-1)./(F_true_mean-1))^.5;
pi_scaling_grid=min_scaling:(max_scaling-min_scaling)/(pi_grid_points-1):max_scaling;


draw_base=randn(instruments*2,sims);
zeta_base=randn(instruments*2,RB_draws);
QZ=ZZ;
ZZ_rt=ZZ^.5;

Bias_2SLS=zeros(length(rho_grid),length(pi_scaling_grid));
MAD_2SLS=zeros(length(rho_grid),length(pi_scaling_grid));
Bias_FULL=zeros(length(rho_grid),length(pi_scaling_grid));
MAD_FULL=zeros(length(rho_grid),length(pi_scaling_grid));
Bias=zeros(length(rho_grid),length(pi_scaling_grid));
MAD=zeros(length(rho_grid),length(pi_scaling_grid));
Oracle_bias=zeros(length(rho_grid),length(pi_scaling_grid));
Oracle_MAD=zeros(length(rho_grid),length(pi_scaling_grid));
Bias_c=zeros(length(rho_grid),length(pi_scaling_grid),length(c_grid));
MAD_c=zeros(length(rho_grid),length(pi_scaling_grid),length(c_grid));
RB_quantile_store=zeros(length(quantile_grid),length(pi_scaling_grid),length(rho_grid));
FULL_quantile_store=zeros(length(quantile_grid),length(pi_scaling_grid),length(rho_grid));
LIML_quantile_store=zeros(length(quantile_grid),length(pi_scaling_grid),length(rho_grid));
TSLS_quantile_store=zeros(length(quantile_grid),length(pi_scaling_grid),length(rho_grid));
Oracle_quantile_store=zeros(length(quantile_grid),length(pi_scaling_grid),length(rho_grid));
RB_t_quantile_store=zeros(length(quantile_grid),length(pi_scaling_grid),length(c_grid),length(rho_grid));
         
for m0=1:length(rho_grid)
    
    beta_true=0;
    rho=rho_grid(m0);
    sigmaU=1;
    sigmaV=1;
    sigmaUV=sigmaU*sigmaV*rho;
    
    temp=[sigmaU^2 rho*sigmaU*sigmaV; rho*sigmaU*sigmaV sigmaV^2];
    Sigma=kron(temp,ZZ_inv);
    Sigma12=rho*sigmaU*sigmaV*ZZ_inv;
    Sigma22=sigmaV^2*ZZ_inv;
    
    sigma12_vec=diag(Sigma12);
    sigma22_vec=diag(Sigma22).^.5;
    sigma2_tilde=2^.5*sigma22_vec;
    sigma12_tilde=2*sigma12_vec;
    sigma2_tilde_mat=repmat(sigma2_tilde,1,RB_draws);
    sigma12_tilde_mat=repmat(sigma12_tilde,1,RB_draws);
    
    draw=Sigma^.5*draw_base;
    zeta=Sigma^.5*zeta_base;
    
    %construct matricies useful for estimation based on transformations of the
    %instruments
    w2_store=zeros(instruments,instruments,length(c_grid));
    w2_inv_store=zeros(instruments,instruments,length(c_grid));
    sigma2t_mat=zeros(instruments,RB_draws,length(c_grid));
    sigma12t_mat=zeros(instruments,RB_draws,length(c_grid));
    for m2=1:length(c_grid)
        c=c_grid(m2);
        A_c=ones(instruments,instruments)*c+eye(instruments)*(1-c);
        temp=diag(diag(Sigma22))^-.5*Sigma22*diag(diag(Sigma22))^-.5;
        w2_store(:,:,m2)=diag(diag(A_c*temp*A_c))^-.5*A_c*diag(diag(Sigma22))^-.5;
        w2_inv_store(:,:,m2)=w2_store(:,:,m2)^-1;
        w2=w2_store(:,:,m2);
        
        sigma2t_vec=diag(2*w2*Sigma22*w2').^.5;
        sigma12t_vec=diag(2*w2*Sigma12*w2');
        sigma2t_mat(:,:,m2)=repmat(sigma2t_vec,1,RB_draws);
        sigma12t_mat(:,:,m2)=repmat(sigma12t_vec,1,RB_draws);
    end
    
    
    %Construct matrices useful in calculation of oracle estimator
    A=(kron(eye(2),pi_true)'*Sigma^-1*kron(eye(2),pi_true))^-1*...
        kron(eye(2),pi_true)'*Sigma^-1;
    Sigma_star=(kron(eye(2),pi_true)'*Sigma^-1*kron(eye(2),pi_true))^-1;
    sigma2_star=Sigma_star(2,2)^.5;
    sigma12_star=Sigma_star(1,2);
    
    
    beta_hat_RB_store=zeros(sims,length(pi_scaling_grid));
    beta_hat_oracle_store=zeros(sims,length(pi_scaling_grid));
    beta_hat_RB_t_store=zeros(sims,length(pi_scaling_grid),length(c_grid));
    beta_hat_2SLS_store=zeros(sims,length(pi_scaling_grid));
    beta_hat_LIML_store=zeros(sims,length(pi_scaling_grid));
    beta_hat_FULL_store=zeros(sims,length(pi_scaling_grid));
    for m=1:length(pi_scaling_grid)
        pi_val=pi_true*pi_scaling_grid(m);
        parfor n=1:sims
        %for n=1:sims
            xi=draw(:,n)+[pi_val*beta_true;pi_val];
            
            xi1=xi(1:instruments,:);
            xi2=xi(instruments+1:end,:);
            beta_hat_2SLS_store(n,m)=sum((ZZ_rt*xi2).*(ZZ_rt*xi1),1)./sum((ZZ_rt*xi2).^2,1);
            
            r1=xi1'*QZ*xi1;
            r2=xi2'*QZ*xi2;
            r3=xi1'*QZ*xi2;
            R=[r1 r3; r3 r2];
            C=[sigmaU^2 sigmaUV; sigmaUV sigmaV^2];
            C_inv_rt=C^-.5;
            kappa=-min(eig(C_inv_rt*R*C_inv_rt)); 
            
            beta_hat_LIML_store(n,m)=(r3+kappa*sigmaUV)/(r2+kappa*sigmaV^2);
            beta_hat_FULL_store(n,m)=(r3+(kappa+1)*sigmaUV)/(r2+(kappa+1)*sigmaV^2);
            
            
            xi_a=repmat(xi,1,RB_draws)+zeta;
            xi_b=repmat(xi,1,RB_draws)-zeta;
            xi1_a=xi_a(1:instruments,:);
            xi2_a=xi_a(instruments+1:end,:);
            xi2_b=xi_b(instruments+1:end,:);
            tau=pi^.5./(2^.5*sigma2_tilde_mat).*exp(xi2_a.^2./(2*sigma2_tilde_mat.^2)).*erfc(xi2_a./(2^.5*sigma2_tilde_mat));
            beta_hat=tau.*(xi1_a-sigma12_tilde_mat./sigma2_tilde_mat.^2.*xi2_a)+sigma12_tilde_mat./sigma2_tilde_mat.^2;
            %Substitute 2SLS for unbiased estimator in cases where cannot
            %compute unbiased estimate.
            elementwise_IV_mat=xi1_a./xi2_a;
            beta_hat(isnan(beta_hat))=elementwise_IV_mat(isnan(beta_hat));
            beta_hat(isinf(beta_hat))=elementwise_IV_mat(isinf(beta_hat));
            
            w=(ZZ*xi2_b).*xi2_b;
            w=w./repmat(sum(w,1),instruments,1);
            beta_hat_RB_store(n,m)=mean(sum(w.*beta_hat,1));
            
            %Compute the RB estimators based on linear combinations of the
            %instruments
            beta_hat_RB_t_temp_store=zeros(length(c_grid),1);
            for m2=1:length(c_grid)
                w2=w2_store(:,:,m2);
                w2_inv=w2_inv_store(:,:,m2);
                
                xi1_a_t=w2*xi1_a;
                xi2_a_t=w2*xi2_a;
                xi2_b_t=w2*xi2_b;
                
                tau=pi^.5./(2^.5*sigma2t_mat(:,:,m2)).*exp(xi2_a_t.^2./(2*sigma2t_mat(:,:,m2).^2)).*erfc(xi2_a_t./(2^.5*sigma2t_mat(:,:,m2)));
                beta_hat=tau.*(xi1_a_t-sigma12t_mat(:,:,m2)./(sigma2t_mat(:,:,m2).^2).*xi2_a_t)+sigma12t_mat(:,:,m2)./(sigma2t_mat(:,:,m2).^2);
                w=(w2_inv'*ZZ*w2_inv*xi2_b_t).*(xi2_b_t);
                w=w./repmat(sum(w,1),instruments,1);
                %Substitute 2SLS for unbiased estimator in cases where cannot
                %compute unbiased estimate.
                elementwise_IV_mat=xi1_a_t./xi2_a_t;
                beta_hat(isnan(beta_hat))=elementwise_IV_mat(isnan(beta_hat));
                beta_hat(isinf(beta_hat))=elementwise_IV_mat(isinf(beta_hat));
                beta_hat_RB_t_temp_store(m2)=mean(sum(w.*beta_hat,1));
            end
            
            %Calculate oracle estimator to construct risk bound
            xi_star=A*xi;
            
            tau=pi^.5/(2^.5*sigma2_star)*exp(xi_star(2).^2/(2*sigma2_star^2)).*erfc(xi_star(2)/(2^.5*sigma2_star));
            beta_hat=tau.*(xi_star(1)-sigma12_star/sigma2_star^2*xi_star(2))+sigma12_star/sigma2_star^2;
            if isnan(beta_hat)==1||isinf(beta_hat)==1
                beta_hat=xi_star(1)/xi_star(2);
            end
            beta_hat_oracle_store(n,m)=beta_hat;
            beta_hat_RB_t_store(n,m,:)=permute(beta_hat_RB_t_temp_store,[3 2 4 1]);
        end
        F_stat_mean(m,1)=pi_val'*Sigma22^-1*pi_val/instruments+1;
        
    end
    Bias_2SLS(m0,:)=mean(beta_hat_2SLS_store-beta_true,1);
    MAD_2SLS(m0,:)=mean(abs(beta_hat_2SLS_store-beta_true),1);
    Bias_FULL(m0,:)=mean(beta_hat_FULL_store-beta_true,1);
    MAD_FULL(m0,:)=mean(abs(beta_hat_FULL_store-beta_true),1);
    Bias(m0,:)=mean(beta_hat_RB_store-beta_true,1);
    MAD(m0,:)=mean(abs(beta_hat_RB_store-beta_true),1);
    Oracle_bias(m0,:)=mean(beta_hat_oracle_store-beta_true,1);
    Oracle_MAD(m0,:)=mean(abs(beta_hat_oracle_store-beta_true),1);
    
    for m2=1:length(c_grid)
        Bias_c(m0,:,m2)=mean(beta_hat_RB_t_store(:,:,m2)-beta_true,1);
        MAD_c(m0,:,m2)=mean(abs(beta_hat_RB_t_store(:,:,m2)-beta_true),1);
    end
    
    for m1=1:length(pi_scaling_grid)
        RB_quantile_store(:,m1,m0)=quantile(beta_hat_RB_store(:,m1),quantile_grid);
        FULL_quantile_store(:,m1,m0)=quantile(beta_hat_FULL_store(:,m1),quantile_grid);
        LIML_quantile_store(:,m1,m0)=quantile(beta_hat_LIML_store(:,m1),quantile_grid);
        TSLS_quantile_store(:,m1,m0)=quantile(beta_hat_2SLS_store(:,m1),quantile_grid);
        Oracle_quantile_store(:,m1,m0)=quantile(beta_hat_oracle_store(:,m1),quantile_grid);
        for m2=1:length(c_grid)
            RB_t_quantile_store(:,m1,m2,m0)=quantile(beta_hat_RB_t_store(:,m1,m2),quantile_grid);
        end
    end
    
end

Bias_Oracle=Oracle_bias;
MAD_Oracle=Oracle_MAD;



clear sigma12t_mat sigma12_tilde_mat sigma2_tilde_mat sigma2t_tilde_mat ...
    draw tau beta_hat beta_hat_RB_store beta_hat_FULL_store beta_hat_LIML_store beta_hat_2SLS_store  ...
    beta_hat_oracle_store zeta sigma2t_mat A B Sigma11 Sigma12 sigma12_star sigma12_tilde...
    sigma12_vec sigma12t_vec Sigma21 Sigma22 sigma22_vec sigma2_star sigma2_tilde...
    sigma2_vec sigma2t_vec Omega11 Omega12 Omega21 Omega22 Omega w2_store w2_inv_store...
    beta_hat_RB_t_store draw_base zeta_base

save Risk_sim_results_10Msims_SS3
exit