%Calculate bias of Fuller and unbiased estimators via numerical integration
clear

sigma1=1;
sigma2=1;
rho=0.95;
sigma12=sigma1*sigma2*rho;
beta=0;

Omega=[sigma1^2 sigma12;...
    sigma12 sigma2^2];
pi_grid=[0.05:.05:2].^2;

mean_beta_hat=NaN(length(pi_grid),1);
mean_beta_hat_Full=NaN(length(pi_grid),1);
for n=1:length(pi_grid)
    pi_n=pi_grid(n);
    sigma_delta=(sigma1^2-sigma12*sigma2^-2*sigma12)^.5;
    f=@(x,y) (2*pi)^-.5*sigma2^-1*exp(-.5*sigma2^-1*(x-pi_n).^2).*...
        (2*pi)^-.5*sigma_delta^-1.*...
        exp(-.5*sigma_delta^-2*(y-sigma12/sigma2^2*x-(pi_n*beta-sigma12/sigma2^2*pi_n)).^2);
    tau=@(x) pi^.5/(2^.5*sigma2)*exp(x.^2/(2*sigma2^2)).*erfc(x/(2^.5*sigma2));
    beta_hat=@(x,y) tau(x).*(y-sigma12/sigma2^2*x)+sigma12/sigma2^2;
    beta_hat_Full=@(x,y) (y.*x+sigma12)./(x.^2+sigma2^2);
    
    gU=@(x,y) beta_hat(x,y).*f(x,y);
    gFull=@(x,y) beta_hat_Full(x,y).*f(x,y);
    
    F_mean(n,1)=pi_n^2+1;
    lowerx=-30;
    upperx=30;
    lowery=-30;
    uppery=30;
    mean_beta_hat(n,1)=integral2(gU,lowerx,upperx,lowery,uppery,'Method','iterated',...
'AbsTol',1e-14,'RelTol',1e-14);
    mean_beta_hat_Full(n,1)=integral2(gFull,lowerx,upperx,lowery,uppery,'Method','iterated',...
'AbsTol',1e-14,'RelTol',1e-14);
end

if rho==0.1
    save Univariate_mean_rho1
elseif rho==0.5
    save Univariate_mean_rho5    
elseif rho==0.95
    save Univariate_mean_rho95    
end